// DbCompiler.cpp: implementation of the CDbCompiler class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "netcontrol.h"
#include "BuildFCCDbDlg.h"
#include "DbCompiler.h"
#include "MainFrm.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


BOOL CreateCompilerProgressBar(	CProgressCtrl* pProgressCtrl,
							CStatusBar* pStatusBar,
							LPCTSTR szMessage = NULL,
							int nPaneIndex = 0,
							int cxMargin = 10,
							int cxMaxWidth = -1,
							UINT nIDControl = 1 )
{
	ASSERT_VALID( pProgressCtrl );
	ASSERT_VALID( pStatusBar );

	// 1 - Calculate destination rectangle for progress control
    CRect rc;
    pStatusBar->GetItemRect( nPaneIndex, &rc );

	// 2 - Define progress bar horizontal offset
	if( szMessage != NULL )
	{
		// Compute message text extent
		CClientDC dc( pStatusBar );
		CFont* pFont = pStatusBar->GetFont();
		CFont* pOldFont = dc.SelectObject( pFont );
		CSize sizeText = dc.GetTextExtent( szMessage );
		dc.SelectObject( pOldFont );

		rc.left += sizeText.cx + cxMargin;
	}

	// 3 - Compute progress bar width
	if( cxMaxWidth != -1 )
	{
		rc.right = rc.left + min( cxMaxWidth, rc.Width() );
	}

	// 4 - Display message text
	pStatusBar->SetPaneText( nPaneIndex, szMessage );
	pStatusBar->RedrawWindow();

	// 5 - Create progress control
    return pProgressCtrl->Create(	WS_CHILD | WS_VISIBLE,
									rc, pStatusBar, nIDControl );
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDbCompiler::CDbCompiler()
{
   CMainFrame* pFrame = (CMainFrame*) AfxGetApp()->m_pMainWnd;
   CreateCompilerProgressBar(&m_wndProgress, &pFrame->m_wndStatusBar, NULL, 1 );
   m_wndProgress.SetStep( 1 );
   m_wndProgress.SetRange( 0, 100 );
   m_wndProgress.SetBitmap(IDB_PROGRESS);
   m_wndProgress.SetPos(0);
}

CDbCompiler::~CDbCompiler()
{

}

void CDbCompiler::execute()
{
   if(m_szJobToDo.Compare("AMSet") == 0)
   {
      PopulateAMDatabase();
   }
   if(m_szJobToDo.Compare("ENSet") == 0)
   {
      PopulateENDatabase();
   }
   if(m_szJobToDo.Compare("HDSet") == 0)
   {
      PopulateHDDatabase();
   }
}

bool CDbCompiler::PopulateAMDatabase()
{
   CFile cf;
   char *lpbuf = NULL;
   int iNameLoc = -1, nCount = 0;
   CString szInputFile, szMsg, szFormatted;
   bool bSuccess = false;
   m_wndProgress.SetPos(0);

   // Pull the AM data out of its file here!
   iNameLoc = m_szInputFolder.ReverseFind('\\');
   szInputFile = m_szInputFolder.Mid(0,iNameLoc) + "\\AM.dat";

   if(!cf.Open(szInputFile, CFile::modeRead))
   {
      AfxMessageBox("Couldn't open the AM.dat file:\n"
                    "Please make sure it is in selected directory, and try again.");
      return false;
   }

   nCount = cf.GetLength()+1;
   lpbuf = (char*) new char[nCount];
   cf.Read(lpbuf,nCount);
   lpbuf[nCount-1] = '\0';
   m_szAMData = lpbuf;
   delete [] lpbuf;
   
   cf.Close();
   
   //  AM Data Parsed here!
   if(m_pAMSet->IsOpen())
   {
      int iLoc = 0, iPos = m_szAMData.Find(0x0a, iLoc);
      while (iPos >= 0)
      {
         CString szRecord(m_szAMData.Mid(iLoc,iPos-iLoc));
         if(!szRecord.IsEmpty())
         {
            int nPercentage = (float)(((float)iLoc/(float)nCount))*(float)100.0;
            szMsg.Format("AM (1 of 3) Build Current position %d%%\n", nPercentage);
            m_pDlg->SetStatus(szMsg);
            //TRACE(szMsg);
            m_wndProgress.SetPos(nPercentage);
            try {
               if(szRecord.GetLength() > 18)
                  ParseAMRecord(szRecord);
            } catch (...) {}
         }
         iLoc = iPos + 1;
         iPos = m_szAMData.Find(0x0a, iLoc);
         bSuccess = true;
      }
   }

   return bSuccess;
}

bool CDbCompiler::ParseAMRecord(CString szRecord)
{
   int idx = 0;
   bool bSuccess = true;
   int iLoc = 0, iPos = szRecord.Find("|", iLoc);

   char* szFieldNames[] = {
         "RecordType", "UniqueSystemIdentifier", "ULSFileNumber", "EBFNumber", 
         "CallSign", "OperatorClass", "GroupCode", "RegionCode", "TrusteeCallSign", 
         "TrusteeIndicator", "PhysicianCertification", "VESignature", 
         "SystematicCallSignChange", "VanityCallSignChange", "VanityRelationship", 
         "PreviousCallSign", "PreviousOperatorClass", "TrusteeName" };
   // try  {
      m_pAMSet->AddNew();
      while (iPos >= 0)
      {
         CString szFieldData(szRecord.Mid(iLoc, iPos-iLoc));
         
         if(!szFieldData.IsEmpty())
         {
            LPCTSTR lpctstr = szFieldData.GetBuffer(0);
            m_pAMSet->SetFieldValue(szFieldNames[idx], lpctstr);
         }

         iLoc = iPos + 1;
         iPos = szRecord.Find("|", iLoc);
         idx++;
      }
      m_pAMSet->Update();
      TRACE ("Added AM Record: %d records parsed\n", idx);

   /* } catch (CDaoException e) {

      LPSTR lpszError = new char[100];
      e.GetErrorMessage(lpszError, 100);
      delete lpszError;
      CString szErr(lpszError);
      AfxMessageBox(szErr);

      bSuccess = false;

   }*/ /* catch (CException e) {

      LPSTR lpszError = new char[100];
      e.GetErrorMessage(lpszError, 100);
      delete lpszError;
      CString szErr(lpszError);
      AfxMessageBox(szErr);

      bSuccess = false;
   }*/
   
   return bSuccess;
}

bool CDbCompiler::PopulateENDatabase()
{
   CFile cf;
   char *lpbuf = NULL;
   int iNameLoc = -1, nCount = 0;
   CString szInputFile, szMsg, szFormatted;
   bool bSuccess = false;
   m_wndProgress.SetPos(0);

   // Pull the EN data out of its file here!
   iNameLoc = m_szInputFolder.ReverseFind('\\');
   szInputFile = m_szInputFolder.Mid(0,iNameLoc) + "\\EN.dat";
   
   if(!cf.Open(szInputFile, CFile::modeRead))
   {
      AfxMessageBox("Couldn't open the EN.dat file:\n"
                    "Please make sure it is in selected directory, and try again.");
      return bSuccess;
   }

   nCount = cf.GetLength()+1;
   lpbuf = (char*) new char[nCount];
   cf.Read(lpbuf,nCount);
   lpbuf[nCount-1] = '\0';
   m_szENData = lpbuf;
   delete [] lpbuf;
   
   cf.Close();	

   //  EN Data Parsed here!
   if(m_pENSet->IsOpen())
   {
      int iLoc = 0, iPos = m_szENData.Find(0x0a, iLoc);
      while (iPos >= 0)
      {
         CString szRecord(m_szENData.Mid(iLoc,iPos-iLoc));
         if(!szRecord.IsEmpty())
         {
            int nPercentage = (float)(((float)iLoc/(float)nCount))*(float)100.0;
            szMsg.Format("EN (2 of 3) Build Current position %d%%\n", nPercentage);
            m_pDlg->SetStatus(szMsg);
            //TRACE(szMsg);
            m_wndProgress.SetPos(nPercentage);
            try {
               if(szRecord.GetLength() > 27)
                  ParseENRecord(szRecord);
            } catch (...) {}
         }
         iLoc = iPos + 1;
         iPos = m_szENData.Find(0x0a, iLoc);
         bSuccess = true;
      }
   }

   return bSuccess;
}

bool CDbCompiler::ParseENRecord(CString szRecord)
{
   int idx = 0;
   bool bSuccess = true;
   int iLoc = 0, iPos = szRecord.Find("|", iLoc);

   char* szFieldNames[] = {
         "RecordType", "UniqueSystemIdentifier", "ULSFileNumber", "EBFNumber", 
         "CallSign", "EntityType", "LicenseID", "EntityName", "FirstName", 
         "MI", "LastName", "Suffix", "Phone", "Fax", "Email", "StreetAddress", 
         "City", "State", "ZipCode", "POBox", "AttentionLine", "SGIN", "FRN", 
         "ApplicantTypeCode", "ApplicantTypeCodeOther", "StatusCode", "StatusDate" };
   // try {
      m_pENSet->AddNew();
      while (iPos >= 0)
      {
         CString szFieldData(szRecord.Mid(iLoc, iPos-iLoc));
         
         if(!szFieldData.IsEmpty())
         {
            LPCTSTR lpctstr = szFieldData.GetBuffer(0);
            m_pENSet->SetFieldValue(szFieldNames[idx], lpctstr);
         }

         iLoc = iPos + 1;
         iPos = szRecord.Find("|", iLoc);
         idx++;
      }
      m_pENSet->Update();

   /* } catch (CDaoException e) {
      LPSTR lpszError = new char[100];
      e.GetErrorMessage(lpszError, 100);
      CString szErr(lpszError);
      AfxMessageBox(szErr);
      delete lpszError;

      bSuccess = false;

   } catch (CException e) {
      LPSTR lpszError = new char[100];
      e.GetErrorMessage(lpszError, 100);
      delete lpszError;
      CString szErr(lpszError);
      AfxMessageBox(szErr);

      bSuccess = false;
   }*/
   
   return bSuccess;
}

bool CDbCompiler::PopulateHDDatabase()
{
   CFile cf;
   char *lpbuf = NULL;
   int iNameLoc = -1, nCount = 0;
   CString szInputFile, szMsg, szFormatted;
   bool bSuccess = false;
   m_wndProgress.SetPos(0);

   // Pull the HD data out of its file here!
   iNameLoc = m_szInputFolder.ReverseFind('\\');
   szInputFile = m_szInputFolder.Mid(0,iNameLoc) + "\\HD.dat";
   
   if(!cf.Open(szInputFile, CFile::modeRead))
   {
      AfxMessageBox("Couldn't open the HD.dat file:\n"
                    "Please make sure it is in selected directory, and try again.");
      return bSuccess;
   }

   nCount = cf.GetLength()+1;
   lpbuf = (char*) new char[nCount];
   cf.Read(lpbuf,nCount);
   lpbuf[nCount-1] = '\0';
   m_szHDData = lpbuf;
   delete [] lpbuf;
   
   cf.Close();	

   //  HD Data Parsed here!
   if(m_pHDSet->IsOpen())
   {
      int iLoc = 0, iPos = m_szHDData.Find(0x0a, iLoc);
      while (iPos >= 0)
      {
         CString szRecord(m_szHDData.Mid(iLoc,iPos-iLoc));
         if(!szRecord.IsEmpty())
         {
            int nPercentage = (float)(((float)iLoc/(float)nCount))*(float)100.0;
            szMsg.Format("HD (3 of 3) Build Current position %d%%\n", nPercentage);
            m_pDlg->SetStatus(szMsg);
            //TRACE(szMsg);
            m_wndProgress.SetPos(nPercentage);
            try {
               if(szRecord.GetLength() > 50)
                  ParseHDRecord(szRecord);
            } catch (...) {}
         }
         iLoc = iPos + 1;
         iPos = m_szHDData.Find(0x0a, iLoc);
         bSuccess = true;
      }
   }

   return bSuccess;
}

bool CDbCompiler::ParseHDRecord(CString szRecord)
{
   int idx = 0;
   bool bSuccess = true;
   int iLoc = 0, iPos = szRecord.Find("|", iLoc);

   char* szFieldNames[] = {
         "RecordType", "UniqueSystemIdentifier", "ULSFileNumber", "EBFNumber", "CallSign",
         "LicenseStatus", "RadioServiceCode", "GrantDate", "ExpiredDate", "CancellationDate",
         "EligibilityRuleNum", "Reserved1", "Alien", "AlienGovernment", "AlienCorporation",
         "AlienOfficer", "AlienControl", "Revoked", "Convicted", "Adjudged", "Reserved2",
         "CommonCarrier", "NonCommonCarrier", "PrivateComm", "Fixed", "Mobile",
         "Radiolocation", "Satellite", "DevelopmentalorSTA", "InterconnectedService",
         "CertifierFirstName", "CertifierMI", "CertifierLastName", "CertifierSuffix",
         "CertifierTitle", "Female", "BlackorAfricanAmerican", "NativeAmerican", "Hawaiian",
         "Asian", "White", "Hispanic", "EffectiveDate", "LastActionDate", "AuctionID",
         "BroadcastServices1", "BandManager", "BroadcastServices2", "AlienRuling",
         "LicenseeNameChange" };
   // try {
      m_pHDSet->AddNew();
      while (iPos >= 0)
      {
         CString szFieldData(szRecord.Mid(iLoc, iPos-iLoc));
         
         if(!szFieldData.IsEmpty())
         {
         LPCTSTR lpctstr = szFieldData.GetBuffer(0);
            m_pHDSet->SetFieldValue(szFieldNames[idx], lpctstr);
         }

         iLoc = iPos + 1;
         iPos = szRecord.Find("|", iLoc);
         idx++;
      }
      m_pHDSet->Update();

   /* } catch (CDaoException e) {

      LPSTR lpszError = new char[100];
      e.GetErrorMessage(lpszError, 100);
      delete lpszError;
      CString szErr(lpszError);
      AfxMessageBox(szErr);

      bSuccess = false;

   } catch (CException e) {

      LPSTR lpszError = new char[100];
      e.GetErrorMessage(lpszError, 100);
      delete lpszError;
      CString szErr(lpszError);
      AfxMessageBox(szErr);

      bSuccess = false;
   }*/
   
   return bSuccess;
}

