// DownloadDlg.cpp : implementation file
//

#include "stdafx.h"
#include "netcontrol.h"
#include "DownloadDlg.h"
#include "FileDialogST.h"
#include "MainFrm.h"
#include "MD5.h"
#include "afxinet.h"

#include <process.h>
#include <direct.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


BOOL CreateDnldProgressBar(	CProgressCtrl* pProgressCtrl,
							CStatusBar* pStatusBar,
							LPCTSTR szMessage = NULL,
							int nPaneIndex = 0,
							int cxMargin = 10,
							int cxMaxWidth = -1,
							UINT nIDControl = 1 )
{
	ASSERT_VALID( pProgressCtrl );
	ASSERT_VALID( pStatusBar );

	// 1 - Calculate destination rectangle for progress control
    CRect rc;
    pStatusBar->GetItemRect( nPaneIndex, &rc );

	// 2 - Define progress bar horizontal offset
	if( szMessage != NULL )
	{
		// Compute message text extent
		CClientDC dc( pStatusBar );
		CFont* pFont = pStatusBar->GetFont();
		CFont* pOldFont = dc.SelectObject( pFont );
		CSize sizeText = dc.GetTextExtent( szMessage );
		dc.SelectObject( pOldFont );

		rc.left += sizeText.cx + cxMargin;
	}

	// 3 - Compute progress bar width
	if( cxMaxWidth != -1 )
	{
		rc.right = rc.left + min( cxMaxWidth, rc.Width() );
	}

	// 4 - Display message text
	pStatusBar->SetPaneText( nPaneIndex, szMessage );
	pStatusBar->RedrawWindow();

	// 5 - Create progress control
    return pProgressCtrl->Create(	WS_CHILD | WS_VISIBLE,
									rc, pStatusBar, nIDControl );
}

/////////////////////////////////////////////////////////////////////////////
// CDownloadDlg dialog

CDownloadDlg::CDownloadDlg(CView* pView)
{
	//{{AFX_DATA_INIT(CDownloadDlg)
   m_szFccFile = _T("l_amat.zip");
   m_szFccPath = _T("/pub/uls/complete");
   m_szFccFullPath.Format(_T("%s/%s"), m_szFccPath, m_szFccFile);
   m_szFccServer = _T("wirelessftp.fcc.gov");
   m_szFccUrl.Format(_T("ftp://%s%s"), m_szFccServer, m_szFccFullPath);

	m_szSaveLocation = _T("C:\\Amateur Radio");

   m_pView = pView;
	//}}AFX_DATA_INIT
   m_tmp_data_file = FCFileEx::QueryTempFilePath() ;
}

BOOL CDownloadDlg::Create()
{
    return CDialog::Create(CDownloadDlg::IDD);
}

void CDownloadDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDownloadDlg)
	DDX_Control(pDX, IDC_DLNLD_PROGRESS, m_ctrlProgress);
	DDX_Text(pDX, IDC_FCC_URL, m_szFccUrl);
	DDX_Text(pDX, IDC_SAVE_LOCATION, m_szSaveLocation);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDownloadDlg, CDialog)
	//{{AFX_MSG_MAP(CDownloadDlg)
	ON_BN_CLICKED(IDC_DOWNLOAD_BUT, OnDownloadBut)
	ON_BN_CLICKED(IDC_SAVE_TO, OnSaveTo)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDownloadDlg message handlers

void CDownloadDlg::OnDownloadBut() 
{
   CNetControlApp* pApp = (CNetControlApp*)AfxGetApp();

   CString sCause, szSaveFilePath;
   sCause.Format("");
   CWaitCursor cursor;

   CInternetSession mysession;
   CFtpConnection* pFtpConn = mysession.GetFtpConnection(m_szFccServer);

   int i = 0;

   if (pFtpConn)
   {
      for (i = 0; i < 50; i++) {
         m_ctrlProgress.SetPos(i);
      }
      //we have successfully connected
      //try to find the remote file 

      CFtpFileFind finder(pFtpConn);

      BOOL bWorking = finder.FindFile(m_szFccFullPath);

      if (bWorking)
      {
         // it exist .. so let's get it
         CFileStatus cfStatus;
         UpdateData(TRUE);
         pApp->WriteProfileString(_T("Settings"), _T("StorageLocation"), m_szSaveLocation);
         szSaveFilePath.Format(_T("%s\\%s"), m_szSaveLocation, m_szFccFile);
         BOOL bStatus = CFile::GetStatus(szSaveFilePath, cfStatus);
         if (bStatus) {
            CFile::Remove(szSaveFilePath);
         }

         BOOL bGot = pFtpConn->GetFile(m_szFccFullPath, szSaveFilePath);

         if (bGot) {
            for (i = 50; i < 100; i++) {
               m_ctrlProgress.SetPos(i);
            }
         }
      }
      else
      {
         //something wrong with the file.. inform the user
         sCause.Format("The File specified could not be found. "
            "Check your spelling and try again!!");
      }
   } else {
      //could not connect
      sCause.Format("Could not connect to ftp server!! \nPlease "
         "ensure you have correct address and access rights!");
   }

   m_ctrlProgress.SetPos(100);

}

void CDownloadDlg::OnCancel() 
{
    // Do not call base class OnCancel
    UpdateData(TRUE);
    if (m_pView != NULL) {
      m_pView->PostMessage(WM_GOODBYE_DNLD, IDCANCEL);
    }
}

void CDownloadDlg::OnOK() 
{
   DeleteAllDownload();

   // Do not call base class OnOK
   UpdateData(TRUE);
   if (m_pView != NULL) {
      m_pView->PostMessage(WM_GOODBYE_DNLD, IDOK);
   }
}

CString g_strTmpPath;

int CALLBACK DNloadBrowseCallbackProc(HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData)
{
	TCHAR szDir[MAX_PATH];
	switch(uMsg){
	case BFFM_INITIALIZED:
		if (lpData){
			strcpy(szDir, g_strTmpPath.GetBuffer(g_strTmpPath.GetLength()));
			SendMessage(hwnd,BFFM_SETSELECTION,TRUE,(LPARAM)szDir);
		}
		break;
	case BFFM_SELCHANGED: {
	   if (SHGetPathFromIDList((LPITEMIDLIST) lParam ,szDir)){
		  SendMessage(hwnd,BFFM_SETSTATUSTEXT,0,(LPARAM)szDir);
	   }
	   break;
	}
	default:
	   break;
	}
         
	return 0;
}

BOOL CDownloadDlg::GetFolder(CString* strSelectedFolder,
				   const char* lpszTitle,
				   const HWND hwndOwner, 
				   const char* strRootFolder, 
				   const char* strStartFolder)
{
	char pszDisplayName[MAX_PATH];
	LPITEMIDLIST lpID;
	BROWSEINFOA bi;
	
	bi.hwndOwner = hwndOwner;
	if (strRootFolder == NULL){
		bi.pidlRoot = NULL;
	}else{
	   LPITEMIDLIST  pIdl = NULL;
	   IShellFolder* pDesktopFolder;
	   char          szPath[MAX_PATH];
	   OLECHAR       olePath[MAX_PATH];
	   ULONG         chEaten;
	   ULONG         dwAttributes;

	   strcpy(szPath, (LPCTSTR)strRootFolder);
	   if (SUCCEEDED(SHGetDesktopFolder(&pDesktopFolder)))
	   {
		   MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, szPath, -1, olePath, MAX_PATH);
		   pDesktopFolder->ParseDisplayName(NULL, NULL, olePath, &chEaten, &pIdl, &dwAttributes);
		   pDesktopFolder->Release();
	   }
	   bi.pidlRoot = pIdl;
	}
	bi.pszDisplayName = pszDisplayName;
	bi.lpszTitle = lpszTitle;
	bi.ulFlags = BIF_RETURNONLYFSDIRS | BIF_STATUSTEXT;
	bi.lpfn = DNloadBrowseCallbackProc;
	if (strStartFolder == NULL){
		bi.lParam = FALSE;
	}else{
		g_strTmpPath.Format("%s", strStartFolder);
		bi.lParam = TRUE;
	}
	bi.iImage = NULL;
	lpID = SHBrowseForFolderA(&bi);
	if (lpID != NULL){
		BOOL b = SHGetPathFromIDList(lpID, pszDisplayName);
		if (b == TRUE){
			strSelectedFolder->Format("%s",pszDisplayName);
			return TRUE;
		}
	}else{
		strSelectedFolder->Empty();
	}
	return FALSE;
}

void CDownloadDlg::OnSaveTo() 
{
	CString strFolderPath, strFolderName;
   CNetControlApp* pApp = (CNetControlApp*)AfxGetApp();

   char szPath[512];
   _getcwd(szPath,_MAX_PATH);

	if (GetFolder(&strFolderPath, "Select a Folder...", this->m_hWnd, NULL, szPath)) {
		if (!strFolderPath.IsEmpty())
      {
         pApp->WriteProfileString(_T("Settings"),  _T("StorageLocation"), strFolderPath);
         m_szSaveLocation = strFolderPath;
         UpdateData(FALSE);
		}
	}
}

BOOL CDownloadDlg::OnInitDialog() 
{
   CNetControlApp* pApp = (CNetControlApp*)AfxGetApp();
	CDialog::OnInitDialog();
	
	m_ctrlProgress.SetRange(0,100);

   m_szSaveLocation = pApp->GetProfileString(_T("Settings"),  _T("StorageLocation"), "");

   if(m_szSaveLocation.IsEmpty())
   {
      char szPath[512];
      _getcwd(szPath,_MAX_PATH);
      m_szSaveLocation.Format(szPath);
   }

	UpdateData(FALSE);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}