// Job.h: interface for the CJob class.
//Author : Pradeep K Sahu
//Email : sahupk@yahoo.com
//Note   : You are free to use the code, and modify. Provided you
//         don't remove this comments. It will be great if you can provide
//		   your feedback.
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_JOB_H__83646D59_DBA8_4E75_9746_0F1014D15313__INCLUDED_)
#define AFX_JOB_H__83646D59_DBA8_4E75_9746_0F1014D15313__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//This class wraps a job. The job that can be added to the queue must be 
//derived from this class. And the method execute() must be over ridden. 
//The function execute() will be called by the the executer.
class CJob
{
public:
	void SetJobName(CString szJobName);
	CString GetJobName();
	CString m_szJobName;
	//This attribute represents the state of the CJob. If the job gets completed 
	//then the flag is TRUE. Else the value is FALSE.
	BOOL m_Completed;

	//A static variable to maintain the last used ID. 
	//So that next ID can be generated by incrementing this. 
	static long lastUsedID;

	//This methods sets the Priority of the Job. 
	void setPriority(int priority);

	//This method returns the priority of the Job. 
	int getPriority();
	
	//This method returns the job ID.
	long getID();
	
	//This method sets the auto delete flag either TRUE or FALSE depending on
	//the parameter. 
	void setAutoDelete(BOOL autoDeleteFlag=TRUE);

	//This method returns TRUE if the autodelete flag is TRUE. Else returns FALSE
	BOOL AutoDelete();

	//The pure virtual method. This method will be implemented by the child classes.
	//This is the method which is the entry point of the Job. 
	virtual void execute()=0;
	
	//Constructor
	CJob();
	
	virtual ~CJob();
private:
	//This is the Job ID. This ID is initialized to the ID in the constructor 
	//of the class. 
	long m_ID;

	//This is a BOOL attribute to represent whether the instance of CJob should be
	//deleted or not after the job gets completed. If TRUE the instance gets deleted 
	//automatically, so the user don't have to call delete again. By default this flag
	//is TRUE.	
	BOOL m_autoDeleteFlag;
	
	//This int attribute represent the priority of the thread. The default value is 5.
	int m_priority;

};

#endif // !defined(AFX_JOB_H__83646D59_DBA8_4E75_9746_0F1014D15313__INCLUDED_)
