// WebRequest.cpp: implementation of the CWebRequest class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "NetControl.h"
#include "NetControlDoc.h"
#include "WebRequest.h"
#include "ProgressCtrlST.h"
#include "MainFrm.h"

#include <sys/timeb.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CWebRequest::CWebRequest()
{
   m_pInetSession = NULL;
   m_pHttpConnection = NULL;
   //m_pInetSession = new CInternetSession("NetControl", 1, PRE_CONFIG_INTERNET_ACCESS);
   m_pInetSession = new CInternetSession(NULL, 1, INTERNET_OPEN_TYPE_PRECONFIG);

   m_pDoc = NULL;
   m_pFile = NULL;
   m_bVerbose = true;
}

CWebRequest::~CWebRequest()
{
   delete m_pInetSession;
}

///////////////////////////////////////////////////////////
// CreateProgressBar

BOOL CreateProgressBar(	CProgressCtrl* pProgressCtrl,
							CStatusBar* pStatusBar,
							LPCTSTR szMessage = NULL,
							int nPaneIndex = 0,
							int cxMargin = 10,
							int cxMaxWidth = -1,
							UINT nIDControl = 1 )
{
	ASSERT_VALID( pProgressCtrl );
	ASSERT_VALID( pStatusBar );

	// 1 - Calculate destination rectangle for progress control
    CRect rc;
    pStatusBar->GetItemRect( nPaneIndex, &rc );

	// 2 - Define progress bar horizontal offset
	if( szMessage != NULL )
	{
		// Compute message text extent
		CClientDC dc( pStatusBar );
		CFont* pFont = pStatusBar->GetFont();
		CFont* pOldFont = dc.SelectObject( pFont );
		CSize sizeText = dc.GetTextExtent( szMessage );
		dc.SelectObject( pOldFont );

		rc.left += sizeText.cx + cxMargin;
	}

	// 3 - Compute progress bar width
	if( cxMaxWidth != -1 )
	{
		rc.right = rc.left + min( cxMaxWidth, rc.Width() );
	}

	// 4 - Display message text
	pStatusBar->SetPaneText( nPaneIndex, szMessage );
   pStatusBar->RedrawWindow();

	// 5 - Create progress control
    return pProgressCtrl->Create(	WS_CHILD | WS_VISIBLE,
									rc, pStatusBar, nIDControl );
}

CString CWebRequest::SubmitRequest(CString szRequest)
{
   /////////////////////////////
   // Progress bar stuff 
   int iStepper = 0;
   CMainFrame* pFrame = (CMainFrame*)AfxGetApp()->m_pMainWnd;

   CProgressCtrlST wndProgress;

   if (m_bVerbose) {
      CreateProgressBar(&wndProgress, &pFrame->m_wndStatusBar, NULL, 1);
      wndProgress.SetStep(4);
      wndProgress.SetRange(0, 500);
      wndProgress.SetBitmap(IDB_PROGRESS);
   }
	//pFrame->m_wndStatusBar.SetWindowText( _T( "Sending Message, press Esc to cancel..." ) );
	//pFrame->m_wndStatusBar.RedrawWindow();

   //////////////////////////////////////
   // Start the retrieval process 
   CString szWebPage, szMsg;
   CString strServerName, strObject;

   DWORD dwValue;
   DWORD dwOption = INTERNET_OPTION_CONNECT_TIMEOUT;


   m_pInetSession->QueryOption(dwOption,dwValue);
   BOOL bSuccess = m_pInetSession->SetOption(dwOption, m_pDoc->GetNetConTO());
   m_pInetSession->QueryOption(dwOption,dwValue);

   dwOption = INTERNET_OPTION_CONNECT_RETRIES;

   m_pInetSession->QueryOption(dwOption,dwValue);
   bSuccess = m_pInetSession->SetOption(dwOption, m_pDoc->GetNetRetNum());
   m_pInetSession->QueryOption(dwOption,dwValue);

   dwOption = INTERNET_OPTION_CONTROL_RECEIVE_TIMEOUT;

   m_pInetSession->QueryOption(dwOption,dwValue);
   bSuccess = m_pInetSession->SetOption(dwOption, m_pDoc->GetNetCtrlRecTO());
   m_pInetSession->QueryOption(dwOption,dwValue);

   dwOption = INTERNET_OPTION_CONTROL_SEND_TIMEOUT;

   m_pInetSession->QueryOption(dwOption,dwValue);
   bSuccess = m_pInetSession->SetOption(dwOption, m_pDoc->GetNetCtrlSndTO());
   m_pInetSession->QueryOption(dwOption,dwValue);

   dwOption = INTERNET_OPTION_DATA_RECEIVE_TIMEOUT;

   m_pInetSession->QueryOption(dwOption,dwValue);
   bSuccess = m_pInetSession->SetOption(dwOption, m_pDoc->GetNetDataRecTO());
   m_pInetSession->QueryOption(dwOption,dwValue);

   dwOption = INTERNET_OPTION_DATA_SEND_TIMEOUT;

   m_pInetSession->QueryOption(dwOption,dwValue);
   bSuccess = m_pInetSession->SetOption(dwOption, m_pDoc->GetNetDataSndTO());
   m_pInetSession->QueryOption(dwOption,dwValue);

   dwOption = INTERNET_OPTION_RECEIVE_TIMEOUT;

   m_pInetSession->QueryOption(dwOption,dwValue);
   bSuccess = m_pInetSession->SetOption(dwOption, m_pDoc->GetNetRecTO());
   m_pInetSession->QueryOption(dwOption,dwValue);

   dwOption = INTERNET_OPTION_SEND_TIMEOUT;

   m_pInetSession->QueryOption(dwOption,dwValue);
   bSuccess = m_pInetSession->SetOption(dwOption, m_pDoc->GetNetSndTO());
   m_pInetSession->QueryOption(dwOption,dwValue);
 
   clock_t tStart, tFinish;
      
	DWORD dwServiceType;
	INTERNET_PORT nPort = 80;

   TRACE("OnSubmit: debut\n");

	CString str;

   TRACE("OnSubmit: create internet session\n");

   // Alert the user if the inet session could
	// not be started and close 
	if (!m_pInetSession) {
		return "Not a good session identifier";
	}

   ///////////////////////////////////////
   // Incriment the progress control
   for(iStepper=0; iStepper<25; iStepper++)
   {
      if (m_bVerbose)
         wndProgress.StepIt();
   }

   SetURL();
   TRACE("OnSubmit: parse URL\n");

   if (!AfxParseURL(m_strHttpURL, dwServiceType, strServerName, strObject, nPort))
	{
		// try adding the "http://" protocol
		m_strHttpURL.Insert(0,"http://");

		if (!AfxParseURL(m_strHttpURL, dwServiceType, strServerName, strObject, nPort))
		{
         return "Invalid URL";
		}
	}

   ///////////////////////////////////////
   // Incriment the progress control
   for(iStepper=25; iStepper<50; iStepper++)
   {
      if (m_bVerbose)
         wndProgress.StepIt();
   }

   if(m_bVerbose)
      CWaitCursor cursor; // this will automatically display a wait cursor

   TRACE("OnSubmit: get http connection\n");

	if (((dwServiceType == INTERNET_SERVICE_HTTP) || (dwServiceType == AFX_INET_SERVICE_HTTPS))  && !strServerName.IsEmpty())
	{
		try
		{
			m_pHttpConnection = m_pInetSession->GetHttpConnection(strServerName,nPort,m_szUserName,m_szPassword);
		}
		catch (CInternetException* pEx)
		{
			// catch errors from WinINet
			TCHAR szErr[1024];
			if (pEx->GetErrorMessage(szErr, 1024))
				AfxMessageBox(szErr, MB_OK);
			else
				AfxMessageBox("Exception Occured", MB_OK);
			pEx->Delete();

			m_pHttpConnection = NULL;
		}
   } else {
		AfxMessageBox("Invalid URL", MB_OK);
	}

   ///////////////////////////////////////
   // Incriment the progress control
   for(iStepper=50; iStepper<75; iStepper++)
   {
      if (m_bVerbose)
         wndProgress.StepIt();
   }

   DWORD dwHttpRequestFlags = NULL;
   dwHttpRequestFlags = INTERNET_FLAG_RELOAD |
                        INTERNET_FLAG_DONT_CACHE |
                        INTERNET_FLAG_NO_AUTO_REDIRECT;


   const TCHAR szHeaders[] =
      _T("Accept: text/*\r\nUser-Agent: NetControlRequest\r\n");


   TRACE("OnSubmit: open connection\n");

   if (m_pHttpConnection != NULL)
   {
      //StatusMsg("Got Connect, Sending Request");
      //CString strHeaders = _T("Content-Type: application/x-www-form-urlencoded");
      CString strHeaders = _T("Content-Type: text/html");

      int      nVerb            = CHttpConnection::HTTP_VERB_GET;
      LPCTSTR  pstrReferer      = NULL;
      LPCTSTR* pstrAcceptTypes  = NULL;
         //(const char **)"image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, application/x-shockwave-flash, application/vnd.ms-excel, application/vnd.ms-powerpoint, application/msword, */*";
         //(const char **) _T("text/*");
      DWORD    dwContext        = 1;
      LPCTSTR  pstrVersion      = _T("HTTP/1.0");
      
      m_pFile = m_pHttpConnection->OpenRequest(nVerb,
                                             strObject,
                                             pstrReferer,
                                             dwContext,
                                             pstrAcceptTypes,
                                             pstrVersion,
                                             dwHttpRequestFlags);

      //m_pHttpConnection->SetOption(
      
      TRACE("OnSubmit: send request\n");

      tStart = clock();

      // m_pFile->SetOption SetClientCertificate
	  BOOL result = m_pFile->SendRequest();

      //BOOL result = m_pFile->SendRequest( strHeaders, 
      //               (LPVOID)(LPCTSTR) szRequest,
      //                         (DWORD) szRequest.GetLength() ); 
      
      DWORD dwRet;
      m_pFile->QueryInfoStatusCode(dwRet);

		if (dwRet == HTTP_STATUS_DENIED)

		{
			DWORD dwPrompt;
			dwPrompt = m_pFile->ErrorDlg(NULL, ERROR_INTERNET_INCORRECT_PASSWORD,
				FLAGS_ERROR_UI_FLAGS_GENERATE_DATA | FLAGS_ERROR_UI_FLAGS_CHANGE_OPTIONS, NULL);

			// if the user cancelled the dialog, bail out

			if (dwPrompt != ERROR_INTERNET_FORCE_RETRY)
			{
            //StatusMsg();
				return "Access denied: Invalid password";
			}

			m_pFile->SendRequest();
			m_pFile->QueryInfoStatusCode(dwRet);
		}

      TRACE("OnSubmit: read string\n");

		TCHAR sz[4097];
      UINT uiLength = m_pFile->GetLength();

		while (m_pFile->ReadString(sz, 24))
		{
         if(m_bVerbose)
            wndProgress.StepIt();
         szWebPage += sz;
         iStepper+=10;
         if (m_bVerbose)
            wndProgress.UpdateData();
		}

      TRACE("OnSubmit: close\n");

	  tFinish = clock();

	  for(; iStepper<500; iStepper+=10)
      {
        if(m_bVerbose)
            wndProgress.StepIt();
      }

      if(m_pFile != NULL)
      {
         m_pFile->Close();
         delete m_pFile;
         m_pFile = NULL;
      }

   }

   TRACE("OnSubmit: cleanup\n----------------\n");
   Cleanup();

   m_tElapsed_time = (double)(tFinish - tStart) / CLOCKS_PER_SEC;
   szMsg.Format("\nPost took %5.3f seconds.", m_tElapsed_time );
   if(m_bVerbose)
	   pFrame->m_wndStatusBar.SetPaneText( 1, szMsg);

   return szWebPage;
}

void CWebRequest::SetURL()
{
   if(m_szPort.IsEmpty())
      m_strHttpURL.Format("http://%s/%s", m_szServer, m_szProcessor);
   else
	   m_strHttpURL.Format("http://%s:%s/%s", m_szServer, m_szPort, m_szProcessor);
}

void CWebRequest::Cleanup()
{
   if(m_pHttpConnection != NULL) {
      m_pHttpConnection->Close();
      delete m_pHttpConnection;
      m_pHttpConnection = NULL;
   }
}

void CWebRequest::SetServer(CString szServer)
{
   m_szServer = szServer;
}

void CWebRequest::SetPort(CString szPort)
{
   m_szPort = szPort;
}

void CWebRequest::SetUserName(CString szUserName)
{
   m_szUserName = szUserName;
}

void CWebRequest::SetProcessor(CString szProcessor)
{
   m_szProcessor = szProcessor;
}

void CWebRequest::SetPassWord(CString szPassWord)
{
   m_szPassword = szPassWord;
}

void CWebRequest::OnTimer()
{
   //m_pFile->EndRequest();
   //Cleanup();
   //AfxEndThread(0);
}

void CWebRequest::SetDocument(CNetControlDoc *pDoc)
{
   m_pDoc = pDoc;
}

double CWebRequest::GetLastElapsedTime()
{
   return m_tElapsed_time;
}
